{{>header}}
{{#standard}}
<h1>{{title}}</h1>
  <table class='stats'>
  {{#stats}}
    <tr>
      <th>
        {{key}}
      </th>
      <td>
        {{value}}
      </td>
    </tr>
  {{/stats}}
  </table>
{{/standard}}
{{#resque_keys}}
<h1>{{title}}</h1>
<p class='sub'>(All keys are actually prefixed with "resque:")</p>
    <table class='stats'>
    <tr>
        <th>key</th>
        <th>type</th>
        <th>size</th>
    </tr>
    {{#stats}}
    <tr>
        <th>
        <a href="/stat/{{key}}">{{key}}</a>
        </th>
        <td>{{type}}</td>
        <td>{{size}}</td>
    </tr>
  {{/stats}}
  </table>
{{/resque_keys}}
{{>footer}}