{{>header}}

<h1>Pending jobs on <span class='hl'>{{queue}}</span></h1>
  <p class='sub'>Showing {{start}} to {{end}} of <b>{{size}}</b> jobs</p>
  <ul class='pagination'>
  {{#pagination}}
      <li>
          {{#current}}<a href='{{link}}'>{{/current}}{{link_name}}{{#current}}</a>{{/current}}
      </li>
  {{/pagination}}
  </ul>
  <table class='jobs'>
    <tr>
      <th>Class</th>
      <th>Args</th>
    </tr>
    {{#jobs}}
    <tr>
      <td class='class'>{{class}}</td>
      <td class='args'>{{args}}</td>
    </tr>
    {{/jobs}}
    {{#empty_job}}
    <tr>
      <td class='no-data' colspan='2'>There are no pending jobs in this queue</td>
    </tr>
    {{/empty_job}}
  </table>
{{>footer}}