<!DOCTYPE html>
<html>
<head>
  <title>pyres.</title>
  <link href="{{media_folder}}reset.css" media="screen" rel="stylesheet" type="text/css">
  <link href="{{media_folder}}style.css" media="screen" rel="stylesheet" type="text/css">
  <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.4.0/jquery.min.js" type="text/javascript"></script>
</head>
<body>
  <div class="header">
    <ul class='nav'>
        <li><a href='/'>Overview</a></li>
        <li><a href='/working/'>Working</a></li>
        <li><a href='/failed/'>Failed</a></li>
        <li><a href='/queues/'>Queues</a></li>
        <li><a href='/workers/'>Workers</a></li>
        <li><a href='/delayed/'>Delayed</a></li>
        <li><a href='/stats/'>Stats</a></li>
    </ul>
  </div>
  
  <ul class='subnav'>
  {{#sub_nav}}
        <li ><a href="/{{section}}/{{subtab}}/"><span>{{subtab}}</span></a></li>
  {{/sub_nav}}
  </ul>
  
<div id="main">
